/**
 * @file ios的网络请求拦截post请求会丢失body，
 *       因此需要hook xhr、fetch及form，若是post请求，则通知native的net hook，关闭拦截
 *       对于ios10以上（存在Proxy对象），使用proxy拦截xhr的open和send方法
 *       对于ios10以下，使用wrapper对象包装xhr
 *       对于fetch使用wrapper进行包装
 *       对于form。由于难以拦截submit，仅考虑插入到dom树中的情形，使用 mutationObserver 观察body元素的改变，
 *       如果发现存在 method=POST 的form则关闭拦截
 *       注意由于 mutationObserver 特性，理论上无法拦截创建后直接提交的form
 *       以上关闭拦截后均不执行打开的行为
 */

/* eslint-disable */
; (function (
    w // for shorter text
) {
    var XHRNAME = 'XMLHttpRequest';
    var FETCHNAME = 'fetch';
    var oldXHR = w[XHRNAME];
    var oldFetch = w[FETCHNAME];
    var NS = '__inner_xhr_hook__';

    var DEBUG = false;
    var callbackHandlers = w[NS] = {};

    function regist(callback) {
        var name = '_test_' + (new Date()).getTime() + (Math.random() * 1000000).toFixed(0);
        callbackHandlers[name] = function () {
            callbackHandlers[name] = undefined;
            callback();
        };
        return name;
    }

    function hookSend(callback) {
        var start = new Date().getTime();
        DEBUG && console.log('before send');
        if (w.webkit && w.webkit.messageHandlers) {

            var name = regist(function () {
                var end = new Date().getTime();
                console.log( end - start);
                callback();
            });

            w.webkit.messageHandlers
                .invokeBdWalletNative.postMessage(
                    [{
                        successcallback: NS + '.' + name,
                        errorcallback: NS + '.' + name,
                        options: { method_name: 'closeOffline' }
                    }]);
        } else {
            callback();
        }

    }

    function extendWithProxy() {
        var handler = {
            construct: function (target) {
                var obj = new target();
                var oldSend = obj.send;
                var oldOpen = obj.open;
                var method = 'get';
                var aysnc = true;

                obj.open = function (_method, url, _async) {
                    DEBUG && console.log('xhr open', arguments);
                    method = (_method + '').toLowerCase();
                    if (arguments.length > 2) {
                        async = !!_async;
                    }
                    oldOpen.apply(obj, [].slice.call(arguments));
                };
                obj.send = function (args) {
                    if (method == 'post') {
                        hookSend(function () {
                            DEBUG && console.log('post callback');
                            oldSend.call(obj, args);
                        });
                    } else {
                        oldSend.call(obj, args);
                    }
                };
                return obj;
            }
        };
        var proxy = new Proxy(oldXHR, handler);
        return proxy;
    }

    function extendFetch() {
        return function (input, init) {
            return new Promise(function (resolve, reject) {
                if (init && (init.method + '').toLowerCase() === 'post') {
                    hookSend(function () {
                        resolve(oldFetch(input, init));
                    });
                } else {
                    resolve(oldFetch(input, init));
                }
            });
        }
    }

    function extendWithWrapper() {
        return function () {
            //创建真正的XMLHttpRequest实例
            var xhr = new oldXHR();
            var self = this;
            var method = 'get';
            var aysnc = true;

            for (var attr in xhr) {
                var type = "";
                try {
                    type = typeof xhr[attr];
                } catch (e) { }
                (function (name, type) {
                    if (type === "function") {
                        //2.代理方法
                        self[name] = function (m, u, a) {
                            var args = [].slice.call(arguments);
                            var d = function (params) {
                                xhr[name].apply(xhr, args);
                            };

                            if (name == 'send') {
                                if (method == 'post') {
                                    hookSend(d);
                                } else {
                                    d();
                                }
                            } else if (name == 'open') {
                                method = (m + '').toLowerCase();
                                if (arguments.length > 2) {
                                    async = !!a;
                                }
                                d();
                            } else {
                                d();
                            }
                        };
                    } else {
                        // 3.代理属性
                        Object.defineProperty(self, name, {
                            get: function () {
                                return xhr[name];
                            },
                            set: function (newVal) {
                                return xhr[name] = newVal;
                            }
                        });
                    }
                }(attr, type));
            }
        };
    }

    if (w.Proxy) {
        // 10以上
        w[XHRNAME] = extendWithProxy();
    } else {
        // 这里得支持一把 8/9
        w[XHRNAME] = extendWithWrapper();
    }

    if (w[FETCHNAME]) {
        w[FETCHNAME] = extendFetch();
    }

    var body = DEBUG ? document.getElementById('main') : document;
    // Create an observer instance linked to the callback function
    var observer = new MutationObserver(function (mutationsList) {
        for (var i = 0; i < mutationsList.length; i++) {
            const element = mutationsList[i];
            DEBUG && console.log('trigger observer', element.type);
            if (body.querySelector('form[method="POST"]')) {
                DEBUG && console.log('trigger observer', 'need close');
                hookSend(function () { });
                // observer.disconnect();
                return;
            }
        }
    }); 

    var config = { attributes: true, childList: true, subtree: true };
    // Start observing the target node for configured mutations
    observer.observe(body, config);

})(window);

